#include <string>

enum Fondness
{
    VeryVague    = 0,
    OftenGood    = 5,
    VeryReliable = 10
};

class Encoding
{
public:
    std::string name;

    virtual ~Encoding() {}

    virtual const Fondness GetFondness() const = 0;

    /* So much code for so little meaning. This is so called purity. */
    class SeqResult
    {
        bool     Bool;
        unsigned Length;
        unsigned Value;
        int ShiftType;
        
    public:
        SeqResult(): Bool(false) { }
        SeqResult(bool v) : Bool(v) { }
        SeqResult(bool v, unsigned len) : Bool(v), Length(len) { }
        SeqResult(bool v, unsigned len, unsigned val) : Bool(v), Length(len), Value(val) { }
        SeqResult(int seq, unsigned len): Bool(true), Length(len), ShiftType(seq) { }
        
        operator bool() const { return Bool; }
        
        unsigned GetLength() const { return Length; }
        unsigned GetValue() const { return Value; }
        int GetShiftType() const { return ShiftType; }
        
        void SetBool(bool b) { Bool = b; }
        void SetLength(unsigned l) { Length = l; }
        void SetValue(unsigned n) { Value = n; }
        void SetShift(int s) { ShiftType = s; }
    };
    
    /* Writing */
    virtual void SeqValue(std::string& result, wchar_t ch, int& shift) const = 0;
    virtual void SeqEnd(std::string& result) const = 0;
    virtual bool Encodable(wchar_t ch, int shift) const = 0;
    
    /* Reading */
    virtual const SeqResult IsShift(const std::string& input, unsigned pos) const = 0;
    virtual const SeqResult IsData(const std::string& input, unsigned pos, int shift) const = 0;
};

/* Find an encoder with given character set name.
 * If not found, throws the value of "name".
 */
const Encoding& FindEncoder(const std::string& name);
